const _ = require("lodash");
const env = require("./config/env");

const express = require("express");
const app = express();
app.use(express.json());

// Config AWS
const AWS = require("aws-sdk");
const credentials = new AWS.SharedIniFileCredentials({
  profile: env.PROFILE,
});
const sns = new AWS.SNS({
  credentials: credentials,
  region: env.REGION,
});

// Routes
app.get("/status", (req, res) => {
  res.status(200).json({
    status: "ok",
    data: sns,
  });
});

// Subscribe to sns topic via email
app.post("/subscribe", (req, res) => {
  let params = {
    Protocol: env.SNS.PROTOCOL,
    TopicArn: env.SNS.TOPIC_ARN,
    Endpoint: req.body.email,
  };

  sns.subscribe(params, (err, data) => {
    if (err) {
      res.status(500).json({
        status: "500",
        err: err,
      });
    } else {
      res.status(200).json({
        status: "ok",
        data: data,
      });
    }
  });
});

// Get subscription status
app.get("/subscription/list", (req, res) => {
  let param = {
    TopicArn: env.SNS.TOPIC_ARN,
  };

  sns.listSubscriptionsByTopic(param, (err, data) => {
    if (err) {
      res.status(500).json({
        status: "500",
        err: err,
      });
    } else {
      // console.log(data);
      let subscriptions = [];
      _.forEach(data.Subscriptions, (ele) => {
        let item = {
          protocol: _.upperCase(ele.Protocol),
          endpoint: censorEmail(ele.Endpoint),
          confirmed: isConfirmed(ele.SubscriptionArn),
        };

        subscriptions.push(item);
      });

      res.status(200).json({
        status: "ok",
        data: subscriptions,
      });
    }
  });
});

// Publish an email notification via sns topic
// all the subscriptions attached to the topic will receive the notification
app.post("/publish", (req, res) => {
  let params = {
    Subject: req.body.subject,
    Message: req.body.message,
    TopicArn: env.SNS.TOPIC_ARN,
  };

  sns.publish(params, function (err, data) {
    if (err) {
      res.status(500).json({
        status: "500",
        err: err,
      });
    } else {
      res.status(200).json({
        status: "ok",
        data: data,
      });
    }
  });
});

// -- private methods --
let censorWord = function (str) {
  return str[0] + "*".repeat(str.length - 2) + str.slice(-1);
};

let censorEmail = function (email) {
  let arr = email.split("@");
  return censorWord(arr[0]) + "@" + censorWord(arr[1]);
};

let isConfirmed = function (status) {
  return _.isEqual(status, "PendingConfirmation") ? false : true;
};

// starting the server
const port = 6000;
app.listen(port, () => {
  console.log(`SNS app listening on port ${port}`);
});
